/*******************************************************************************
 * Copyright 2011 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *******************************************************************************/
package net.ihe.gazelle.proxy.authentication;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import net.ihe.gazelle.proxy.admin.gui.ApplicationConfigurationManager;

import org.jasig.cas.client.authentication.DefaultGatewayResolverImpl;
import org.jasig.cas.client.authentication.GatewayResolver;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.util.CommonUtils;
import org.jasig.cas.client.validation.Assertion;
import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.contexts.Lifecycle;
import org.jboss.seam.log.LogProvider;
import org.jboss.seam.log.Logging;

public class AuthenticationFilter extends AbstractCasFilter {

	private static final LogProvider log = Logging
			.getLogProvider(AuthenticationFilter.class);

	/**
	 * Whether to send the renew request or not.
	 */
	private boolean renew = false;

	/**
	 * Whether to send the gateway request or not.
	 */
	private boolean gateway = false;

	private GatewayResolver gatewayStorage = new DefaultGatewayResolverImpl();

	private boolean isCASEnabled = false;

	/**
	 * The URL to the CAS Server login.
	 */
	private String casServerLoginUrl;

	public AuthenticationFilter() {
		super();

		isCASEnabled = false;
		casServerLoginUrl = "-";

		setIgnoreInitConfiguration(true);
		setArtifactParameterName("ticket");
		setServiceParameterName("service");
		setEncodeServiceUrl(true);
		setServerName(null);
		setService("-");
	}

	@Override
	protected void initInternal(FilterConfig filterconfig)
			throws ServletException {
		super.initInternal(filterconfig);
	}

	@Override
	public void doFilter(ServletRequest servletRequest,
			ServletResponse servletResponse, FilterChain filterChain)
			throws IOException, ServletException {
		reloadParameters();
		if (isCASEnabled) {
			doFilterCAS(servletRequest, servletResponse, filterChain);
		} else {
			filterChain.doFilter(servletRequest, servletResponse);
		}
	}

	private void reloadParameters() {
		// Create a seam context if needed
		boolean createContexts = !Contexts.isEventContextActive()
				&& !Contexts.isApplicationContextActive();
		if (createContexts) {
			Lifecycle.beginCall();
		}

		setService(ApplicationConfigurationManager.instance()
				.getApplicationUrl());
		casServerLoginUrl = ApplicationConfigurationManager.instance()
				.getCasUrl();
		isCASEnabled = !ApplicationConfigurationManager.instance()
				.isWorksWithoutCas();

		if (createContexts) {
			Lifecycle.endCall();
		}
	}

	private void doFilterCAS(ServletRequest servletRequest,
			ServletResponse servletResponse, FilterChain filterChain)
			throws IOException, ServletException {
		final HttpServletRequest request = (HttpServletRequest) servletRequest;
		final HttpServletResponse response = (HttpServletResponse) servletResponse;
		final HttpSession session = request.getSession(false);
		final Assertion assertion = session != null ? (Assertion) session
				.getAttribute(CONST_CAS_ASSERTION) : null;

		if (assertion != null) {
			filterChain.doFilter(request, response);
			return;
		}

		final String serviceUrl = constructServiceUrl(request, response);
		final String ticket = CommonUtils.safeGetParameter(request,
				getArtifactParameterName());
		final boolean wasGatewayed = gatewayStorage.hasGatewayedAlready(
				request, serviceUrl);

		if (CommonUtils.isNotBlank(ticket) || wasGatewayed) {
			filterChain.doFilter(request, response);
			return;
		}

		final String modifiedServiceUrl;

		log.debug("no ticket and no assertion found");
		if (gateway) {
			log.debug("setting gateway attribute in session");
			modifiedServiceUrl = gatewayStorage.storeGatewayInformation(
					request, serviceUrl);
		} else {
			modifiedServiceUrl = serviceUrl;
		}

		if (log.isDebugEnabled()) {
			log.debug("Constructed service url: " + modifiedServiceUrl);
		}

		final String urlToRedirectTo = CommonUtils.constructRedirectUrl(
				this.casServerLoginUrl, getServiceParameterName(),
				modifiedServiceUrl, this.renew, gateway);

		if (log.isDebugEnabled()) {
			log.debug("redirecting to \"" + urlToRedirectTo + "\"");
		}

		response.sendRedirect(urlToRedirectTo);
	}

}
