package net.ihe.gazelle.proxy.netty.protocols.dicom;

import java.io.IOException;

import jp.digitalsensation.ihej.transactionmonitor.dicom.messageexchange.Association;
import jp.digitalsensation.ihej.transactionmonitor.dicom.messageexchange.DimseMessage;
import jp.digitalsensation.ihej.transactionmonitor.dicom.messageexchange.DimseMessageParser;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelStateEvent;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.ExceptionEvent;
import org.jboss.netty.channel.MessageEvent;
import org.jboss.netty.channel.SimpleChannelUpstreamHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DicomDecoder extends SimpleChannelUpstreamHandler {

	static final Logger log = LoggerFactory.getLogger(DicomDecoder.class);
	private DimseMessageParser dimseMessageParser;

	// private PipedOutputStream pos;

	public DicomDecoder(String storageFolder) {
		super();
		try {
			dimseMessageParser = new DimseMessageParser(new Association(),
					storageFolder);
			// PipedInputStream pis = new PipedInputStream();
			// pos = new PipedOutputStream(pis);
			// new Thread(new ProxyAssociation(storageFolder, pis)).run();
		} catch (IOException e) {
			throw new IllegalArgumentException(e);
		}
	}

	@Override
	public void messageReceived(ChannelHandlerContext ctx, MessageEvent e)
			throws Exception {
		Object m = e.getMessage();
		if (!(m instanceof ChannelBuffer)) {
			ctx.sendUpstream(e);
			return;
		}

		ChannelBuffer input = (ChannelBuffer) m;
		if (!input.readable()) {
			return;
		}

		try {
			byte[] buf = new byte[input.readableBytes()];
			input.readBytes(buf);

			Iterable<DimseMessage> messages = dimseMessageParser.push(buf);
			for (DimseMessage dimseMessage : messages) {
				Channels.fireMessageReceived(ctx, dimseMessage);
			}

			// pos.write(buf);
			// List<ProxyDicomMessage> messages =
			// proxyAssociation.getMessages();
			// for (ProxyDicomMessage dicomMessage : messages) {
			// Channels.fireMessageReceived(ctx, dicomMessage);
			// }
		} catch (Exception ex) {
			log.error("", ex);
		}
	}

	@Override
	public void channelUnbound(ChannelHandlerContext ctx, ChannelStateEvent e)
			throws Exception {
		super.channelUnbound(ctx, e);
		// pos.close();
	}

	@Override
	public void channelDisconnected(ChannelHandlerContext ctx,
			ChannelStateEvent e) throws Exception {
		super.channelDisconnected(ctx, e);
		// pos.close();
	}

	@Override
	public void channelClosed(ChannelHandlerContext ctx, ChannelStateEvent e)
			throws Exception {
		super.channelClosed(ctx, e);
		// pos.close();
	}

	@Override
	public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e)
			throws Exception {
		ctx.sendUpstream(e);
	}

}
