package net.ihe.gazelle.proxy.netty.protocols.syslog;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.jboss.netty.buffer.ChannelBuffer;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.handler.codec.frame.FrameDecoder;
import org.openhealthtools.openatna.syslog.SyslogMessage;
import org.openhealthtools.openatna.syslog.SyslogMessageFactory;

class SyslogFrameDecoder extends FrameDecoder {

	private static final int STATUS_INIT = 0;
	private static final int STATUS_SIZE = 1;
	private static final int STATUS_READ = 2;

	private int status = STATUS_INIT;
	private StringBuffer messageSizeBuffer = null;
	private int messageSize;
	private ByteArrayOutputStream currentFrame = null;

	@Override
	protected Object decode(ChannelHandlerContext ctx, Channel channel,
			ChannelBuffer buffer) throws Exception {
		while (buffer.readableBytes() > 0) {
			byte readByte = buffer.readByte();

			if (status == STATUS_INIT) {
				messageSizeBuffer = new StringBuffer();
				currentFrame = new ByteArrayOutputStream();
				status = STATUS_SIZE;
			}

			switch (status) {
			case STATUS_SIZE:
				if (readByte == 32) {
					status = STATUS_READ;
					messageSize = Integer
							.parseInt(messageSizeBuffer.toString());
				} else {
					char c = (char) readByte;
					messageSizeBuffer.append(c);
				}
				break;
			case STATUS_READ:
				currentFrame.write(readByte);
				if (currentFrame.size() == messageSize) {
					status = STATUS_INIT;
					byte[] bytes = currentFrame.toByteArray();
					String rawMessage = new String(bytes, "UTF-8");
					SyslogMessage<?> syslogMessage = SyslogMessageFactory
							.getFactory().read(new ByteArrayInputStream(bytes));
					return new SyslogData(syslogMessage, rawMessage);
				}
				break;
			default:
				break;
			}
		}
		return null;
	}
}
